@@ -23,24 +23,24 @@ import Data.Aeson.Optics.Ext
 import Data.Char (isDigit)
 import Data.Text qualified as T
 import Optics.Core
-import Text.Read (readEither)
+import Text.Read (readMaybe)
 
-data Token = N Int | E | K Key
+data Token = K Key | N Int | E
   deriving stock (Eq, Show)
 
 -- | Access a key or array index like 'ix', used for indexing
 tokenL :: Token -> AffineTraversal' Value Value
 tokenL t = case t of
+  K k -> key k
   N n -> nth n
   E -> atEnd % _Just
-  K k -> key k
 
 -- | Access a key or array index like 'at', used for adding or removing
 atTokenL :: Token -> AffineTraversal' Value (Maybe Value)
 atTokenL = \case
+  K k -> atKey k
   N n -> atNth n
   E -> atEnd
-  K k -> atKey k
 
 tokenFromText :: Text -> Either String Token
 tokenFromText = \case
@@ -55,10 +55,10 @@ tokenFromText = \case
 readDigits :: Text -> Either String Int
 readDigits t
   | t == "0" = Right 0
-  | T.isPrefixOf "0" t = Left "leading zeros"
+  | T.isPrefixOf "0" t = Left "tokens cannot have leading zeros"
   | otherwise =
-      first (\msg -> "could not read digits " <> unpack t <> ": " <> msg)
-        $ readEither
+      note ("could not read " <> show t <> " as integer")
+        $ readMaybe
         $ unpack t
 
 tokenToText :: Token -> Text
